==> dynamodb-utils-4.0.5.tgz: package/src/client/DDBClient.ts <==
import {BatchWriteItemCommand, DeleteItemCommand, PutItemCommand, UpdateItemCommand} from '@aws-sdk/client-dynamodb'
import type {DynamoDBClientConfig} from '@aws-sdk/client-dynamodb/dist-types/DynamoDBClient'
import {unmarshall} from '@aws-sdk/util-dynamodb'
import chunk from 'lodash/chunk'
import {DDBReadOnlyClient} from './DDBReadOnlyClient'
import {formatKey} from './formatter/formatKey'
import {mapCreateItemCommand} from './mapper/mapCreateItemCommand'
import {mapCreateItemsCommand} from './mapper/mapCreateItemsCommand'
import {mapDeleteItemCommand} from './mapper/mapDeleteItemCommand'
import {mapDeleteItemsCommand} from './mapper/mapDeleteItemsCommand'
import {mapUpdateItemCommand} from './mapper/mapUpdateItemCommand'

export interface DDBClientConfiguration<
  Item extends object,
  PKName extends keyof Item,
  SKName extends keyof Item | never = never,
> {
  tableName: string
  partitionKeyName: PKName
  sortKeyName?: SKName
  configuration?: DynamoDBClientConfig
}

export class DDBClient<
  Item extends object,
  PKName extends keyof Item,
  SKName extends keyof Item | never = never,
> extends DDBReadOnlyClient<Item, PKName, SKName> {
  async createItem(item: Item): Promise<Item> {
    let command = undefined
    try {
      const partitionKey = item[this.config.partitionKeyName]
      const sortKey = this.config.sortKeyName ? item[this.config.sortKeyName] : undefined
      this.requireFullPrimaryKey(partitionKey, sortKey)

      command = mapCreateItemCommand(this.config, item)
      await this.client.send(new PutItemCommand(command))
      return item
    } catch (err) {
      console.error('Unable to create item:', JSON.stringify(item), 'with command', JSON.stringify(command))
      throw err
    }
  }

  async createItems(items: Item[]): Promise<void> {
    let command = undefined
    try {
      command = mapCreateItemsCommand(this.config, items)
      await this.client.send(new BatchWriteItemCommand(command))
    } catch (err) {
      console.error('Unable to create items:', JSON.stringify(items), 'with command', JSON.stringify(command))
      throw err
    }
  }

  async updateItem(
    item: Partial<Item> & Record<PKName, Item[PKName]> & Record<SKName, Item[SKName]>,
    createIfNotExists = false,
  ): Promise<Item> {
    const partitionKey = item[this.config.partitionKeyName]
    const sortKey = this.config.sortKeyName && item[this.config.sortKeyName]

    let command = undefined
    try {
      this.requireFullPrimaryKey(partitionKey, sortKey)

      command = mapUpdateItemCommand(this.config, item, createIfNotExists)
      const response = await this.client.send(new UpdateItemCommand(command))

      if (!response.Attributes) throw new Error('Updated item not returned')

      return unmarshall(response.Attributes) as Item
    } catch (error) {
      if (error && typeof error === 'object' && 'name' in error && error?.name === 'ConditionalCheckFailedException') {
        throw new Error(`Can't update item as it does not exist: ${formatKey(this.config, partitionKey, sortKey)}`)
      }

      console.error('Unable to update item:', JSON.stringify(item), 'with command', JSON.stringify(command))
      throw error
    }
  }

  async deleteItem(partitionKey: Item[PKName], sortKey?: Item[SKName]): Promise<void> {
    let command = undefined
    try {
      this.requireFullPrimaryKey(partitionKey, sortKey)

      command = mapDeleteItemCommand(this.config, partitionKey, sortKey)
      await this.client.send(new DeleteItemCommand(command))
    } catch (err) {
      console.error(
        'Unable to delete item:',
        formatKey(this.config, partitionKey, sortKey),
        'with command',
        JSON.stringify(command),
      )
      throw err
    }
  }

  async deleteItems(items: (Record<PKName, Item[PKName]> & Record<SKName, Item[SKName]>)[]): Promise<void> {
    let command = undefined
    try {
      const chunks = chunk(items, 25)
      for (const chunk1 of chunks) {
        if (chunk1.length === 0) return
        command = mapDeleteItemsCommand(this.config, chunk1)
        await this.client.send(new BatchWriteItemCommand(command))
      }
    } catch (err) {
      console.error('Unable to delete items:', JSON.stringify(items), 'with command', JSON.stringify(command))
      throw err
    }
  }
}

==> dynamodb-utils-5.0.0.tgz: package/src/client/DDBClient.ts <==
import {BatchWriteItemCommand, DeleteItemCommand, PutItemCommand, UpdateItemCommand} from '@aws-sdk/client-dynamodb'
import type {DynamoDBClientConfig} from '@aws-sdk/client-dynamodb'
import {unmarshall} from '@aws-sdk/util-dynamodb'
import {chunk} from 'lodash'
import {DDBReadOnlyClient} from './DDBReadOnlyClient.ts'
import {formatKey} from './formatter/formatKey.ts'
import {mapCreateItemCommand} from './mapper/mapCreateItemCommand.ts'
import {mapCreateItemsCommand} from './mapper/mapCreateItemsCommand.ts'
import {mapDeleteItemCommand} from './mapper/mapDeleteItemCommand.ts'
import {mapDeleteItemsCommand} from './mapper/mapDeleteItemsCommand.ts'
import {mapUpdateItemCommand} from './mapper/mapUpdateItemCommand.ts'

export interface DDBClientConfiguration<
  Item extends object,
  PKName extends keyof Item,
  SKName extends keyof Item | never = never,
> {
  tableName: string
  partitionKeyName: PKName
  sortKeyName?: SKName
  configuration?: DynamoDBClientConfig
}

export class DDBClient<
  Item extends object,
  PKName extends keyof Item,
  SKName extends keyof Item | never = never,
> extends DDBReadOnlyClient<Item, PKName, SKName> {
  async createItem(item: Item): Promise<Item> {
    let command = undefined
    try {
      const partitionKey = item[this.config.partitionKeyName]
      const sortKey = this.config.sortKeyName ? item[this.config.sortKeyName] : undefined
      this.requireFullPrimaryKey(partitionKey, sortKey)

      command = mapCreateItemCommand(this.config, item)
      await this.client.send(new PutItemCommand(command))
      return item
    } catch (err) {
      console.error('Unable to create item:', JSON.stringify(item), 'with command', JSON.stringify(command))
      throw err
    }
  }

  async createItems(items: Item[]): Promise<void> {
    let command = undefined
    try {
      command = mapCreateItemsCommand(this.config, items)
      await this.client.send(new BatchWriteItemCommand(command))
    } catch (err) {
      console.error('Unable to create items:', JSON.stringify(items), 'with command', JSON.stringify(command))
      throw err
    }
  }

  async updateItem(
    item: Partial<Item> & Record<PKName, Item[PKName]> & Record<SKName, Item[SKName]>,
    createIfNotExists = false,
  ): Promise<Item> {
    const partitionKey = item[this.config.partitionKeyName]
    const sortKey = this.config.sortKeyName && item[this.config.sortKeyName]

    let command = undefined
    try {
      this.requireFullPrimaryKey(partitionKey, sortKey)

      command = mapUpdateItemCommand(this.config, item, createIfNotExists)
      const response = await this.client.send(new UpdateItemCommand(command))

      if (!response.Attributes) throw new Error('Updated item not returned')

      return unmarshall(response.Attributes) as Item
    } catch (error) {
      if (error && typeof error === 'object' && 'name' in error && error?.name === 'ConditionalCheckFailedException') {
        throw new Error(`Can't update item as it does not exist: ${formatKey(this.config, partitionKey, sortKey)}`)
      }

      console.error('Unable to update item:', JSON.stringify(item), 'with command', JSON.stringify(command))
      throw error
    }
  }

  async deleteItem(partitionKey: Item[PKName], sortKey?: Item[SKName]): Promise<void> {
    let command = undefined
    try {
      this.requireFullPrimaryKey(partitionKey, sortKey)

      command = mapDeleteItemCommand(this.config, partitionKey, sortKey)
      await this.client.send(new DeleteItemCommand(command))
    } catch (err) {
      console.error(
        'Unable to delete item:',
        formatKey(this.config, partitionKey, sortKey),
        'with command',
        JSON.stringify(command),
      )
      throw err
    }
  }

  async deleteItems(items: (Record<PKName, Item[PKName]> & Record<SKName, Item[SKName]>)[]): Promise<void> {
    let command = undefined
    try {
      const chunks = chunk(items, 25)
      for (const chunk1 of chunks) {
        if (chunk1.length === 0) return
        command = mapDeleteItemsCommand(this.config, chunk1)
        await this.client.send(new BatchWriteItemCommand(command))
      }
    } catch (err) {
      console.error('Unable to delete items:', JSON.stringify(items), 'with command', JSON.stringify(command))
      throw err
    }
  }
}
